##############################################################################
#
#    Menu redirector for Odoo
#    Copyright © 2015, 2018, 2019, 2022, 2024 XCG Consulting
#              (https://xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import logging

from odoo import SUPERUSER_ID, registry
from odoo.api import Environment
from odoo.http import Controller, db_filter, request, route
from werkzeug import utils

log = logging.getLogger(__name__)

REDIRECT_MSG = "Redirecting client to configured URL: %s"

NO_MENU_ERROR = "No menu provided: redirecting to /"
NO_APP_ERROR = "No app provided: redirecting to /"
NO_DBNAME_ERROR = "No dbname provided: redirecting to /"


def request_error(dbname, app, menu):
    error = False
    if not dbname:
        log.error(NO_DBNAME_ERROR)
        error = True

    if not app:
        log.error(NO_APP_ERROR)
        error = True

    if not menu:
        log.error(NO_MENU_ERROR)
        error = True

    return error


class MenuRedirector(Controller):
    @route("/menuredir/go", type="http", auth="none")
    def redirect_to_menu(self, **kw):
        """A simple helper that will respond to the /menuredir/go URL.
        Find the correct menu id and redirect the client to the proper URL as if he had
        clicked on the menu.
        """
        app, menu = (
            kw.get("app", None),
            kw.get("menu", None),
        )
        # Inspired by code in auth_oauth controller
        dbname = kw.get("db", None)
        if not dbname:
            dbname = request.db
        if dbname and not db_filter([dbname]):
            # filter out any filtered dbname
            dbname = None

        url = "/web"
        error_ = request_error(dbname, app, menu)
        cache_header = None

        if not error_:
            try:
                odoo_registry = registry(dbname)

                with odoo_registry.cursor() as cr:
                    try:
                        env = Environment(cr, SUPERUSER_ID, {})
                        log.debug(
                            "Searching menu item %s.%s for database %s",
                            app,
                            menu,
                            dbname,
                        )
                        irmd = env["ir.model.data"]
                        # pylint: disable=protected-access
                        res_model, res_id = irmd._xmlid_to_res_model_res_id(
                            f"{app}.{menu}", True
                        )
                        if res_model == "ir.ui.menu":
                            url = (
                                "/web"
                                + (f"?db={dbname}" if "db" in kw else "")
                                + f"#menu_id={res_id}&action="
                            )
                            log.debug(REDIRECT_MSG, url)
                        else:
                            log.error("%s.%s is not a menu xmlid", app, menu)
                        # this almost could be a permanent redirect, but if the menu
                        # entry is deleted (which should not be done), the id would
                        # change
                        # 1 week cache
                        cache_header = "max-age=604800"
                    except ValueError:
                        log.error(
                            "Menu item %s.%s not found in db %s", menu, app, dbname
                        )
                    except Exception:
                        log.exception("Unknown error searching for menu item")

            except Exception:
                log.exception("Invalid db provided: redirecting to /")

        redirect = utils.redirect(url, 302)
        redirect.autocorrect_location_header = True
        if cache_header:
            redirect.headers.add("Cache-Control", cache_header)

        return redirect
