from odoo.tests import HttpCase, tagged
from odoo.tools import mute_logger


@tagged("post_install", "-at_install")
class TestMenuRedirectorController(HttpCase):
    """Tests of the module controller."""

    def test_valid_menu(self):
        """Test the redirect of a valid menu item"""
        dbname = self.env.cr.dbname
        menu = "menu_action_res_users"
        url = f"/menuredir/go?db={dbname}&app=base&menu={menu}"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
        self.assertEqual(res.status_code, 200, "Response should = OK")
        menu_id = self.env.ref(f"base.{menu}").id
        self.assertTrue(res.url.endswith(f"#menu_id={menu_id}&action="))

    def test_missing_db(self):
        """Test the redirect when the database name is missing"""
        menu = "menu_action_res_users"
        url = f"/menuredir/go?app=base&menu={menu}"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
            self.assertEqual(res.status_code, 200, "Response should = OK")
            # Tests are run with a single database, so it should redirect correctly
            menu_id = self.env.ref(f"base.{menu}").id
            self.assertTrue(res.url.endswith(f"#menu_id={menu_id}&action="))

    def test_invalid_db(self):
        """Test the redirect when the database name is incorrect"""
        # Assumes <dbname>-invalid is not a valid database name
        dbname = self.env.cr.dbname + "-invalid"
        menu = "menu_action_res_users"
        url = f"/menuredir/go?db={dbname}&app=base&menu={menu}"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
            self.assertEqual(res.status_code, 200, "Response should = OK")
            self.assertTrue(res.url.endswith("/web/login"))

    def test_missing_app(self):
        """Test the redirect when the module name is missing"""
        dbname = self.env.cr.dbname
        menu = "menu_action_res_users"
        url = f"/menuredir/go?db={dbname}&menu={menu}"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
            self.assertEqual(res.status_code, 200, "Response should = OK")
            self.assertTrue(res.url.endswith("/web/login"))

    def test_messing_menu(self):
        """Test the redirect when the menu xmlid is missing"""
        dbname = self.env.cr.dbname
        url = f"/menuredir/go?db={dbname}&app=base"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
            self.assertEqual(res.status_code, 200, "Response should = OK")
            self.assertTrue(res.url.endswith("/web/login"))

    def test_invalid_menu(self):
        """Test the redirect when the menu xmlid is incorrect"""
        dbname = self.env.cr.dbname
        menu = "made_up_id"
        url = f"/menuredir/go?db={dbname}&app=base&menu={menu}"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
            self.assertEqual(res.status_code, 200, "Response should = OK")
            self.assertTrue(res.url.endswith("/web/login"))

    def test_not_a_menu(self):
        """Test the redirect when the menu xmlid is not a menu"""
        dbname = self.env.cr.dbname
        menu = "group_user"
        url = f"/menuredir/go?db={dbname}&app=base&menu={menu}"
        with mute_logger("odoo.addons.menuredir.controllers.main", "werkzeug"):
            res = self.url_open(url, allow_redirects=False)
            self.assertEqual(res.status_code, 302, "Response should be a redirect")
            res = self.url_open(url, allow_redirects=True)
            self.assertEqual(res.status_code, 200, "Response should = OK")
            self.assertTrue(res.url.endswith("/web/login"))
